NameTT = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceDebug-2.0", "AceConsole-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local Tourist = AceLibrary("Tourist-2.0")
local L = AceLibrary("AceLocale-2.2"):new("NameTT")

local OPT_SHOW = 1
local OPT_HIDE = 0
local OPT_IGNORE = -1

function bool2int(bool)
	if bool then return 1 else return 0 end
end

function NameTT:OnInitialize()


	local options = {
		type = 'group',
		args = {
			title = {
				name = L["Name Tag Toggle"],
				type = 'header',
				order = 1,
			},
			arenas = {
				type = 'toggle',
				name = L["Arenas"], desc = L["Show/hide tags in arenas."],
				get = function() return self.db.profile.tags.arenas end,
				set = function(v) self.db.profile.tags.arenas = v; self:ZONE_CHANGED() end,
				order = 3,
			},
			battlegrounds = {
				type = 'toggle',
				name = L["Battlegrounds"], desc = L["Show/hide tags in battlegrounds."],
				get = function() return self.db.profile.tags.battlegrounds end,
				set = function(v) self.db.profile.tags.battlegrounds = v; self:ZONE_CHANGED() end,
				order = 4,
			},
			instances = {
				type = 'toggle',
				name = L["Instances"], desc = L["Show/hide tags in instances."],
				get = function() return self.db.profile.tags.instances end,
				set = function(v) self.db.profile.tags.instances = v; self:ZONE_CHANGED() end,
				order = 5,
			},
			pvp = {
				type = 'toggle',
				name = L["PvP"], desc = L["Show NameTags when flagged for PvP. When not flagged tags will be displayed based on the zone type."],
				get = function() return self.db.profile.tags.pvp end,
				set = function(v) self.db.profile.tags.pvp = v; self:ZONE_CHANGED() end,
				order = 6,
			},
			friendly = {
				type = 'toggle',
				name = L["Friendly"], desc = L["Show/hide tags in friendly (green) zones."],
				get = function() return self.db.profile.tags.friendly end,
				set = function(v) self.db.profile.tags.friendly = v; self:ZONE_CHANGED() end,
				order = 7,
			},
			contested = {
				type = 'toggle',
				name = L["Contested"], desc = L["Show/hide tags in friendly (green) zones."],
				get = function() return self.db.profile.tags.contested end,
				set = function(v) self.db.profile.tags.contested = v; self:ZONE_CHANGED() end,
				order = 8,
			},
			hostile = {
				type = 'toggle',
				name = L["Hostile"], desc = L["Show/hide tags in hostile (red) zones."],
				get = function() return self.db.profile.tags.hostile end,
				set = function(v) self.db.profile.tags.hostile = v; self:ZONE_CHANGED() end,
				order = 9,
			},
			overrides = {
				name = L["Overrides"], desc=L["Current zone Overrides."], type='group',
				args = {
					show = {
						type = 'toggle',
						name = L["Show"], desc=L["Allways show NameTags in this zone."],
						get = "GetZoneShowOverride",
						set = "SetZoneShowOverride",
					},
					hide = {
						type = "toggle",
						name = L["Hide"], desc=L["Allways hide NameTags in this zone."],
						get = "GetZoneHideOverride",
						set = "SetZoneHideOverride",
					},
				},
			},
		},
	} 

	options.args.config = {
		name = L["Config"], desc=L["Show GUI Configuration Menu"], type="execute", guiHidden=true,
		func = function() dewdrop:Open(UIParent, 'children', function()dewdrop:FeedAceOptionsTable(options) end, 'cursorX', true, 'cursorY', true) end,
	}

	NameTT:RegisterDB("NTTDB")
	NameTT:RegisterDefaults("profile", NTT_DEFAULT_OPTIONS)
	NameTT:RegisterChatCommand({"/ntt","/nametagtoggle"}, options)

end

function NameTT:Initialize()
end

function NameTT:Enable()
end

function NameTT:Disable()
end

function NameTT:OnEnable()

	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_INDOORS", "ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZONE_CHANGED")

	self:ZONE_CHANGED()
end

function NameTT:OnDisable()
	self:UnregisterAllEvents()
end

-- ##########################

function NameTT:ZONE_CHANGED()
	self:Debug("zone changed to:", GetZoneText() )
	local override = self:ZoneOverride()
	if not override then
		self:ToggleTags()
	end
end

function NameTT:ZoneOverride()
	local zoneName = GetZoneText()

	for _, zone in pairs(self.db.profile.overrides.on) do
		if (zone == zoneName) then
			SetCVar("UnitNamePlayer",1)
			return true
		end
	end

	for _, zone in pairs(self.db.profile.overrides.off) do
		if(zone == zoneName) then
			SetCVar("UnitNamePlayer", 0)
			return true
		end
	end

	return false
end

function NameTT:SetZoneHideOverride(v)
	for key, zone in pairs(self.db.profile.overrides.off) do
		if zone == GetZoneText() then table.remove(self.db.profile.overrides.off, key ) end
	end
	if v then table.insert(self.db.profile.overrides.off, GetZoneText() ); NameTT:SetZoneShowOverride(false) end
	self:ZONE_CHANGED()
end

function NameTT:GetZoneHideOverride()
	for _, zone in pairs(self.db.profile.overrides.off) do
		if zone == GetZoneText() then return true end
	end
	return false
end

function NameTT:SetZoneShowOverride(v)
	for key, zone in pairs(self.db.profile.overrides.on) do
		if zone == GetZoneText() then table.remove(self.db.profile.overrides.on, key ) end
	end
	if v then table.insert(self.db.profile.overrides.on, GetZoneText() ); NameTT:SetZoneHideOverride(false) end
	self:ZONE_CHANGED()
end

function NameTT:GetZoneShowOverride()
	for _, zone in pairs(self.db.profile.overrides.on) do
		if zone == GetZoneText() then return true end
	end
	return false
end

function NameTT:ToggleTags()

	local zoneName = GetZoneText()

	-- Check battlegrounds
	local BGStat, _, _, _, _ = GetBattlefieldStatus(1)
	if ( BGStat=="active" ) then
		SetCVar("UnitNamePlayer", bool2int(self.db.profile.tags.battlegrounds) )
		self:Debug("We are in a battleground")
		return true
	end

	-- Check arenas
	isArena, _ = IsActiveBattlefieldArena();
	if (isArena == 1) then
		SetCVar("UnitNamePlayer", bool2int(self.db.profile.tags.arenas) )
		self:Debug("We are in an arena")
		return true
	end

	-- Check instances
	if ( IsInInstance() ) then
		SetCVar("UnitNamePlayer", bool2int(self.db.profile.tags.instances) )
		self:Debug("We are in an instance.")
		return true
	end

	-- Check for PvP flag. This is done after the instance check as instances are 'safe'
	if ( UnitIsPVP("player") and self.db.profile.tags.pvp ) then
		SetCVar("UnitNamePlayer", 1)
		self:Debug("Unit is Flagged for PvP")
		return true
	end

	-- Check zone type
	if Tourist:IsFriendly(zoneName) then
		SetCVar("UnitNamePlayer", bool2int(self.db.profile.tags.friendly))
		self:Debug("Zone is friendly")
	elseif Tourist:IsContested(zoneName) then
		SetCVar("UnitNamePlayer", bool2int(self.db.profile.tags.contested))
		self:Debug("Zone is contested")
	elseif Tourist:IsHostile(zoneName) then
		SetCVar("UnitNamePlayer", bool2int(self.db.profile.tags.hostile))
		self:Debug("Zone is hostile")
	else
		self:Debug("PVP type is", pvpType)
	end

end
